//*****************************************************************************
//
//      Class:          Using
//      Date created:   15 June 2001
//      Copyright:       Kenny Kerr 2002    http://www.kennyandkarin.com
//
//*****************************************************************************

#pragma once

#include <gcroot.h>

//
// The Using class provides an alternative in the C++ language to C#'s using 
// keyword. Its basically an unmanaged wrapper for an IDisposable-derived 
// class. When the Using object goes out of scope it ensures that the object's
// Dispose method gets called. This is kinda like deterministic finalization
// for managed objects!
//
template <typename T>
struct Using : gcroot<T>
{
    Using(T t) :
        gcroot<T>(t)
    {
        // Do nothing
    }

    ~Using()
    {
        T t = operator->();

        if (0 != t)
        {
            static_cast<System::IDisposable*>(t)->Dispose();
        }
    }
};
