//*****************************************************************************
//
//      Class:          WindowsAPI
//      Namespace:      Kerr.IconBrowser
//      Date created:   15 June 2001
//      Copyright:       Kenny Kerr 2002    http://www.kennyandkarin.com
//
//*****************************************************************************

#pragma once

using namespace System::Runtime::InteropServices;

namespace Kerr { namespace IconBrowser
{
    //
    // Contains definitions of common unmanaged Win32 API functions, 
    // constants, and structures. For more information see the Platform SDK
    // documentation.
    //
    __gc class WindowsAPI
    {
        // Can't create WindowsAPI objects.
        WindowsAPI() {}

    public:

    // Types

        typedef unsigned int   DWORD;
        typedef unsigned short USHORT;
        typedef unsigned int   UINT;
        typedef long           LONG;
        typedef unsigned short WORD;
        typedef unsigned char  BYTE;

    // Constants

        static const System::IntPtr RT_ICON         = System::IntPtr(3);
        static const System::IntPtr RT_GROUP_ICON   = System::IntPtr(14);
        static const DWORD LOAD_LIBRARY_AS_DATAFILE = 0x00000002;
        static const NO_ERROR                       = 0;
        static const ERROR_RESOURCE_TYPE_NOT_FOUND  = 1813;

    // Helpers

        static void ThrowLastError()
        {
            throw new System::ComponentModel::Win32Exception();
        }

        static DWORD GetLastError()
        {
            return System::Runtime::InteropServices::Marshal::GetLastWin32Error();
        }

    // Functions

        [DllImport(S"Kernel32.dll", SetLastError=true, CharSet=CharSet::Auto)]
        static void* LoadLibraryEx(System::String* strFileName,
                                   System::IntPtr hFile,
                                   DWORD dwFlags);

        [DllImport("Kernel32.dll", SetLastError=true)]
        static bool FreeLibrary(System::IntPtr hModule);

        [DllImport("Kernel32.dll", SetLastError=true, CharSet=CharSet::Auto)]
        static void* FindResource(System::IntPtr hModule,
                                  System::IntPtr pName,
                                  System::IntPtr pType);

        [DllImport("Kernel32.dll", SetLastError=true)]
        static void* LoadResource(System::IntPtr hModule,
                                  System::IntPtr hResource);

        [DllImport("Kernel32.dll", SetLastError=true)]
        static DWORD SizeofResource(System::IntPtr hModule,
                                    System::IntPtr hResource);

        [DllImport("Kernel32.dll")]
        static void* LockResource(System::IntPtr hGlobal);

        __delegate bool EnumResNameProc(System::IntPtr hModule,
                                        System::IntPtr pType,
                                        System::IntPtr pName,
                                        System::IntPtr param);

        [DllImport("Kernel32.dll", SetLastError=true, CharSet=CharSet::Auto)]
        static bool EnumResourceNames(System::IntPtr hModule,
                                      System::IntPtr pType,
                                      EnumResNameProc* enumFunc,
                                      System::IntPtr param);

        [DllImport("User32.dll", SetLastError=true, CharSet=CharSet::Auto)]
        static void* LoadIcon(System::IntPtr hInstance,
                              System::IntPtr pName);

        [DllImport("Shell32.dll", CharSet=CharSet::Auto)]
        static void* ExtractIcon(System::IntPtr hInstance,
                                 System::String* strFileName,
                                 UINT uiIconIndex);

        static bool IS_INTRESOURCE(System::IntPtr wInteger)
        {
            unsigned int uiInteger = reinterpret_cast<unsigned int>(wInteger.ToPointer());
            return (0 == (uiInteger >> 16));
        }

    // Structures

        __nogc struct BITMAPINFOHEADER
        {
            DWORD biSize;
            LONG  biWidth;
            LONG  biHeight;
            WORD  biPlanes;
            WORD  biBitCount;
            DWORD biCompression;
            DWORD biSizeImage;
            LONG  biXPelsPerMeter;
            LONG  biYPelsPerMeter;
            DWORD biClrUsed;
            DWORD biClrImportant;
        };

        __nogc struct RGBQUAD
        {
            BYTE rgbBlue;
            BYTE rgbGreen;
            BYTE rgbRed;
            BYTE rgbReserved;
        };

        __nogc struct BITMAPINFO
        {
            BITMAPINFOHEADER bmiHeader;
            RGBQUAD          bmiColors[1]; // inline array
        };

        //
        // MEMICONDIRENTRY helps to defines the memory layout of a 
        // RT_GROUP_ICON resource. In particular its wId member indicates the
        // RT_ICON resource for the directory entry. This structure is not 
        // found in the Windows headers.
        //
        #pragma pack(push)
        #pragma pack(2)
        __nogc struct MEMICONDIRENTRY
        {
            BYTE  bWidth;
            BYTE  bHeight;
            BYTE  bColorCount;
            BYTE  bReserved;
            WORD  wPlanes;
            WORD  wBitCount;
            DWORD dwBytesInRes;
            WORD  wId;
        };
        #pragma pack(pop)

        //
        // MEMICONDIR defines the memory layout of a RT_GROUP_ICON Win32
        // resource. This structure is not found in the Windows headers.
        //
        #pragma pack(push)
        #pragma pack(2)
        __nogc struct MEMICONDIR
        {
            WORD            wReserved;
            WORD            wType;
            WORD            wCount;
            MEMICONDIRENTRY arEntries[1]; // inline array
        };
        #pragma pack(pop)

        //
        // FILEICONDIRENTRY defines the peristent format of an icon directory 
        // entry in a .ICO file. This structure is not found in the Windows 
        // headers.
        //
        __nogc struct FILEICONDIRENTRY
        {
            BYTE  bWidth;
            BYTE  bHeight;
            BYTE  bColorCount;
            BYTE  bReserved;
            WORD  wPlanes;
            WORD  wBitCount;
            DWORD dwBytesInRes;
            DWORD dwImageOffset;
        };
    };
}}